from __future__ import annotations

import unittest

import numpy as np
import numpy.testing as npt
import ray

import pymwm
from pymwm.slit.samples import Samples, SamplesForRay


class TestSlitSamples(unittest.TestCase):
    def setUp(self):
        self.betas = np.array(
            [
                [
                    1.3376125743759562 + 0.011564042849255995j,
                    0.0027442447191681966 + 10.375841268671248j,
                    0.00097855383808055987 + 20.896897655103722j,
                    0.00011573994545347484 + 31.385614129413867j,
                    -0.00087380515924567834 + 41.866423036789818j,
                    -0.0023972370044439842 + 52.343328327620277j,
                ],
                [
                    2 * np.pi / 10,
                    0.16144417878734335 + 9.052520291531513j,
                    0.3382796371471994 + 18.186379931569725j,
                    0.5625919070749846 + 27.18042875785826j,
                    0.9111719350975946 + 35.945442096320754j,
                    1.732101248998277 + 44.2903211828822j,
                ],
            ]
        )
        self.convs = np.array(
            [
                [True, True, True, True, True, True],
                [False, True, True, True, True, True],
            ]
        )
        self.params = {
            "core": {"shape": "slit", "size": 0.3, "fill": {"RI": 1.0}},
            "clad": {"book": "Au", "page": "Stewart-DLF", "bound_check": False},
            "bounds": {"wl_max": 5.0, "wl_min": 1.0, "wl_imag": 50.0},
            "modes": {
                "wl_max": 5.0,
                "wl_min": 1.0,
                "wl_imag": 50.0,
                "dw": 1.0 / 64,
                "num_n": 6,
            },
        }
        pymwm.create(self.params)

    def test_attributes(self):
        params: dict = self.params.copy()
        r = params["core"]["size"]
        fill = params["core"]["fill"]
        clad = params["clad"]
        wg = Samples(r, fill, clad, params["modes"])
        p = params["modes"]
        ind_w_min = int(np.floor(2 * np.pi / p["wl_max"] / p["dw"]))
        ind_w_max = int(np.ceil(2 * np.pi / p["wl_min"] / p["dw"]))
        ind_w_imag = int(np.ceil(2 * np.pi / p["wl_imag"] / p["dw"]))
        ws = np.arange(ind_w_min, ind_w_max + 1) * p["dw"]
        wis = -np.arange(ind_w_imag + 1) * p["dw"]
        print(ws.shape, wg.ws.shape)
        npt.assert_equal(wg.ws, ws)
        npt.assert_equal(wg.wis, wis)

    @staticmethod
    def beta2_pec(w: complex, n: np.ndarray, e1: complex, r: float) -> np.ndarray:
        h2: np.ndarray = e1 * w ** 2 - (n * np.pi / r) ** 2
        return h2

    def test_beta2_pec(self):
        import riip

        params: dict = self.params.copy()
        r = params["core"]["size"]
        fill = params["core"]["fill"]
        clad = params["clad"]
        print(params["modes"])
        wg = Samples(r, fill, clad, params["modes"])
        w = 2 * np.pi / 5.0
        pec = self.beta2_pec(
            w, np.arange(6), riip.Material(fill)(w), params["core"]["size"]
        )
        h2_e = wg.beta2_pec(w, "even", 6)
        h2_o = wg.beta2_pec(w, "odd", 6)
        h2 = []
        for i in range(3):
            h2 += [h2_e[i], h2_o[i]]
        npt.assert_allclose(h2, pec)

    def test_beta2_w_min(self):
        params: dict = self.params.copy()
        r = params["core"]["size"]
        fill = params["core"]["fill"]
        clad = params["clad"]
        wg = Samples(r, fill, clad, params["modes"])
        self.assertEqual(wg.ws[0], 1.25)
        num_n = params["modes"]["num_n"]
        ray.shutdown()
        try:
            ray.init()
            p_modes_id = ray.put(params["modes"])
            pool = ray.util.ActorPool(
                SamplesForRay.remote(r, fill, clad, p_modes_id) for _ in range(4)
            )
            args = [
                ("M", "even", num_n),
                ("M", "odd", num_n),
                ("E", "even", num_n),
                ("E", "odd", num_n),
            ]
            vals = list(pool.map(lambda a, arg: a.beta2_w_min.remote(*arg), args))
        finally:
            ray.shutdown()
        for i in range(4):
            h2s, success = vals[i]
            for j in range(3):
                self.assertAlmostEqual(h2s[j], self.betas[i // 2][2 * j + i % 2] ** 2)

    def test_db(self):
        params: dict = self.params.copy()
        r = params["core"]["size"]
        fill = params["core"]["fill"]
        clad = params["clad"]
        wg = Samples(r, fill, clad, params["modes"])
        betas, convs = wg.database.load()
        for n in range(6):
            npt.assert_allclose(
                [betas[("M", n, 1)][0, 0], betas[("E", n, 1)][0, 0]],
                [self.betas[0][n], self.betas[1][n]],
            )
            self.assertEqual(
                [convs[("M", n, 1)][0, 0], convs[("E", n, 1)][0, 0]],
                [self.convs[0][n], self.convs[1][n]],
            )

    def test_interpolation(self):
        params: dict = self.params.copy()
        r = params["core"]["size"]
        fill = params["core"]["fill"]
        clad = params["clad"]
        wg = Samples(r, fill, clad, params["modes"])
        betas, convs = wg.database.load()
        vs = {}
        for key in betas.keys():
            h2s = betas[key] ** 2
            ws = wg.ws
            vs[key] = np.array([wg.v(h2, w, wg.clad(w)) for h2, w in zip(h2s, ws)])
        beta_funcs = wg.database.interpolation(
            betas, convs, vs, bounds={"wl_max": 3.0, "wl_min": 1.0, "wl_imag": 50.0}
        )
        self.assertAlmostEqual(
            beta_funcs[(("M", 0, 1), "real")](2 * np.pi, 0.0)[0, 0], 6.7809313363538424
        )
        self.assertAlmostEqual(
            beta_funcs[(("M", 0, 1), "imag")](2 * np.pi, 0.0)[0, 0], 0.01839788
        )
        self.assertAlmostEqual(
            beta_funcs[(("M", 1, 1), "real")](2 * np.pi, 0.0)[0, 0], 0.02905019
        )
        self.assertAlmostEqual(
            beta_funcs[(("M", 1, 1), "imag")](2 * np.pi, 0.0)[0, 0], 7.6016237505487076
        )
        self.assertAlmostEqual(
            beta_funcs[(("M", 2, 1), "real")](2 * np.pi, 0.0)[0, 0], 0.00734511
        )
        self.assertAlmostEqual(
            beta_funcs[(("M", 2, 1), "imag")](2 * np.pi, 0.0)[0, 0], 19.703086274534229
        )
        self.assertAlmostEqual(
            beta_funcs[(("M", 3, 1), "real")](2 * np.pi, 0.0)[0, 0], -0.00016907
        )
        self.assertAlmostEqual(
            beta_funcs[(("M", 3, 1), "imag")](2 * np.pi, 0.0)[0, 0], 30.64071297
        )
        self.assertAlmostEqual(
            beta_funcs[(("E", 1, 1), "real")](2 * np.pi, 0.0)[0, 0], 0.05963503
        )
        self.assertAlmostEqual(
            beta_funcs[(("E", 1, 1), "imag")](2 * np.pi, 0.0)[0, 0], 6.5393800946369307
        )
        self.assertAlmostEqual(
            beta_funcs[(("E", 2, 1), "real")](2 * np.pi, 0.0)[0, 0], 0.09734926449677761
        )
        self.assertAlmostEqual(
            beta_funcs[(("E", 2, 1), "imag")](2 * np.pi, 0.0)[0, 0], 16.952792082725754
        )
        self.assertAlmostEqual(
            beta_funcs[(("E", 3, 1), "real")](2 * np.pi, 0.0)[0, 0], 0.15859938630326686
        )
        self.assertAlmostEqual(
            beta_funcs[(("E", 3, 1), "imag")](2 * np.pi, 0.0)[0, 0], 26.207933276759146
        )
        # self.assertAlmostEqual(
        #     beta_funcs[(('E', 4, 1), 'real')](2 * np.pi, 0.0)[0, 0],
        #     0.26532169)
        # self.assertAlmostEqual(
        #     beta_funcs[(('E', 4, 1), 'imag')](2 * np.pi, 0.0)[0, 0],
        #     34.9463874)



import unittest

import numpy as np
import numpy.testing as npt
import ray

import pymwm
from pymwm.cylinder.samples import Samples, SamplesForRay


class TestCylinderSamples(unittest.TestCase):
    def setUp(self):
        betas = []
        convs = []
        betas.append(
            [
                1.32930242e-03 + 15.97093724j,
                -4.12022699e-04 + 36.77569389j,
                -2.96423554e-03 + 57.67676409j,
                4.25706586e-01 + 22.16438948j,
                1.16961605e00 + 39.91871066j,
            ]
        )
        convs.append([True, True, True, True, True])
        betas.append(
            [
                3.93618487e-04 + 25.50725697j,
                -1.69749464e-03 + 46.75220214j,
                -3.60166883e-03 + 67.81044568j,
                1.88412925e-01 + 10.63429198j,
                6.65409650e-01 + 30.69581722j,
            ]
        )
        convs.append([True, True, True, True, True])
        betas.append(
            [
                -3.14039183e-04 + 34.21067616j,
                -3.02497952e-03 + 56.09987523j,
                -3.99382568e-03 + 77.45436569j,
                3.16581667e-01 + 17.70646046j,
                9.90337935e-01 + 38.34855698j,
            ]
        )
        convs.append([True, True, True, True, True])
        betas.append(
            [
                -1.22828011e-03 + 42.51416161j,
                -3.77291544e-03 + 65.06037821j,
                -4.27041215e-03 + 86.7578285j,
                4.45859022e-01 + 24.35935701j,
                1.56012941e00 + 45.43872731j,
            ]
        )
        convs.append([True, True, True, True, True])
        betas.append(
            [
                -0.00274348 + 50.57304098j,
                -0.00424744 + 73.75302452j,
                -0.00448273 + 95.80756518j,
                0.58332927 + 30.80613956j,
                2.57935560 + 52.37067052j,
            ]
        )
        convs.append([True, True, True, True, True])
        betas.append(
            [
                -0.00422390 + 58.46301045j,
                -0.00458645 + 82.24672285j,
                -0.00465523 + 104.65914944j,
                0.73689393 + 37.1144517j,
                3.79669182 + 59.48095715j,
            ]
        )
        convs.append([True, True, True, True, True])
        self.betas = np.array(betas)
        self.convs = np.array(convs)
        self.params = {
            "core": {"shape": "cylinder", "size": 0.15, "fill": {"RI": 1.0}},
            "clad": {"book": "Au", "page": "Stewart-DLF", "bound_check": False},
            "modes": {
                "wl_max": 5.0,
                "wl_min": 1.0,
                "wl_imag": 50.0,
                "dw": 1.0 / 64,
                "num_n": 6,
                "num_m": 2,
            },
        }

    def test_attributes(self):
        params: dict = self.params.copy()
        size: float = params["core"]["size"]
        fill = params["core"]["fill"]
        clad = params["clad"]
        wg = Samples(size, fill, clad, params["modes"])
        p = params["modes"]
        ind_w_min = int(np.floor(2 * np.pi / p["wl_max"] / p["dw"]))
        ind_w_max = int(np.ceil(2 * np.pi / p["wl_min"] / p["dw"]))
        ind_w_imag = int(np.ceil(2 * np.pi / p["wl_imag"] / p["dw"]))
        ws = np.arange(ind_w_min, ind_w_max + 1) * p["dw"]
        wis = -np.arange(ind_w_imag + 1) * p["dw"]
        npt.assert_equal(wg.ws, ws)
        npt.assert_equal(wg.wis, wis)

    def test_beta2_pec(self):
        params: dict = self.params.copy()
        size: float = params["core"]["size"]
        fill = params["core"]["fill"]
        clad = params["clad"]
        wg = Samples(size, fill, clad, params["modes"])
        w = 2 * np.pi / 5.0
        pec0 = (
            np.array(
                [
                    7.9914227540830467j,
                    18.389529559512987j,
                    28.838915878616223j,
                    12.756889235180587j,
                    23.376846509563137j,
                ]
            )
            * 2
        )
        npt.assert_almost_equal(wg.beta2_pec(w, 0), pec0 ** 2)
        pec1 = (
            np.array(
                [
                    12.756889235180589j,
                    23.376846509563137j,
                    33.90573916009118j,
                    6.1050317506026381j,
                    17.760365196297929j,
                ]
            )
            * 2
        )
        npt.assert_almost_equal(wg.beta2_pec(w, 1), pec1 ** 2)
        pec2 = (
            np.array(
                [
                    17.107206360452462j,
                    28.050444310850633j,
                    38.72770732091869j,
                    10.161382581946896j,
                    22.344945200686148j,
                ]
            )
            * 2
        )
        npt.assert_almost_equal(wg.beta2_pec(w, 2), pec2 ** 2)
        pec3 = (
            np.array(
                [
                    21.257922769420034j,
                    32.530676457118169j,
                    43.37945228513604j,
                    13.989860591754667j,
                    26.710066174072818j,
                ]
            )
            * 2
        )
        npt.assert_almost_equal(wg.beta2_pec(w, 3), pec3 ** 2)
        pec4 = (
            np.array(
                [
                    25.286669814449052j,
                    36.877012636462631j,
                    47.90433520177347j,
                    17.71403733166369j,
                    30.934940730583346j,
                ]
            )
            * 2
        )
        npt.assert_almost_equal(wg.beta2_pec(w, 4), pec4 ** 2)
        pec5 = (
            np.array(
                [
                    29.231527454256089j,
                    41.123881000095977j,
                    52.330141681593268j,
                    21.376155694373626j,
                    35.060573334299526j,
                ]
            )
            * 2
        )
        npt.assert_almost_equal(wg.beta2_pec(w, 5), pec5 ** 2)

    def test_beta2_w_min(self):
        params: dict = {
            "core": {"shape": "cylinder", "size": 0.15, "fill": {"RI": 1.0}},
            "clad": {"book": "Au", "page": "Stewart-DLF", "bound_check": False},
            "modes": {"num_n": 6, "num_m": 2},
        }
        size = params["core"]["size"]
        fill = params["core"]["fill"]
        clad = params["clad"]
        wg = Samples(size, fill, clad, params["modes"])
        self.assertEqual(wg.ws[0], 1.25)
        num_n = params["modes"]["num_n"]
        ray.shutdown()
        try:
            ray.init()
            p_modes_id = ray.put(params["modes"])
            pool = ray.util.ActorPool(
                SamplesForRay.remote(size, fill, clad, p_modes_id) for _ in range(num_n)
            )
            vals = list(
                pool.map(lambda a, arg: a.beta2_w_min.remote(arg), range(num_n))
            )
        finally:
            ray.shutdown()
        for n in range(6):
            print(n)
            h2s, success = vals[n]
            print(success)
            npt.assert_allclose(h2s, self.betas[n] * self.betas[n], rtol=1e-6)

    def test_db(self):
        params: dict = self.params.copy()
        size = params["core"]["size"]
        fill = params["core"]["fill"]
        clad = params["clad"]
        wg = Samples(size, fill, clad, params["modes"])
        try:
            betas, convs = wg.database.load()
        except IndexError:
            num_n = params["modes"]["num_n"]
            ray.shutdown()
            try:
                ray.init()
                p_modes_id = ray.put(params["modes"])
                pool = ray.util.ActorPool(
                    SamplesForRay.remote(size, fill, clad, p_modes_id)
                    for _ in range(num_n)
                )
                xs_success_list = list(
                    pool.map(lambda a, arg: a.task.remote(arg), range(num_n))
                )
            finally:
                ray.shutdown()
            betas, convs = wg.betas_convs(xs_success_list)
            wg.database.save(betas, convs)
        for n in range(6):
            print(n)
            npt.assert_allclose(
                [
                    betas[("M", n, 1)][0, 0],
                    betas[("M", n, 2)][0, 0],
                    betas[("E", n, 1)][0, 0],
                    betas[("E", n, 2)][0, 0],
                ],
                [
                    self.betas[n][0],
                    self.betas[n][1],
                    self.betas[n][3],
                    self.betas[n][4],
                ],
            )
            self.assertEqual(
                [
                    convs[("M", n, 1)][0, 0],
                    convs[("M", n, 2)][0, 0],
                    convs[("E", n, 1)][0, 0],
                    convs[("E", n, 2)][0, 0],
                ],
                [
                    self.convs[n][0],
                    self.convs[n][1],
                    self.convs[n][3],
                    self.convs[n][4],
                ],
            )

    def test_interpolation(self):
        params: dict = self.params.copy()
        size = params["core"]["size"]
        fill = params["core"]["fill"]
        clad = params["clad"]
        wg = Samples(size, fill, clad, params["modes"])
        try:
            betas, convs = wg.database.load()
        except IndexError:
            num_n = params["modes"]["num_n"]
            ray.shutdown()
            try:
                ray.init()
                p_modes_id = ray.put(params["modes"])
                pool = ray.util.ActorPool(
                    SamplesForRay.remote(size, fill, clad, p_modes_id)
                    for _ in range(num_n)
                )
                xs_success_list = list(
                    pool.map(lambda a, arg: a.task.remote(arg), range(num_n))
                )
            finally:
                ray.shutdown()
            betas, convs = wg.betas_convs(xs_success_list)
            wg.database.save(betas, convs)
        vs = {}
        for key in betas.keys():
            h2s = betas[key] ** 2
            ws = wg.ws
            vs[key] = np.array([wg.v(h2, w, wg.clad(w)) for h2, w in zip(h2s, ws)])
        beta_funcs = wg.database.interpolation(
            betas, convs, vs, bounds={"wl_max": 3.0, "wl_min": 1.0, "wl_imag": 100.0}
        )
        self.assertAlmostEqual(
            beta_funcs[(("M", 0, 1), "real")](2 * np.pi, 0.0)[0, 0],
            0.011030829731291485,
        )
        self.assertAlmostEqual(
            beta_funcs[(("M", 0, 1), "imag")](2 * np.pi, 0.0)[0, 0], 14.374412149329419
        )
        self.assertAlmostEqual(
            beta_funcs[(("M", 1, 1), "real")](2 * np.pi, 0.0)[0, 0],
            0.0024442176360349316,
        )
        self.assertAlmostEqual(
            beta_funcs[(("M", 1, 1), "imag")](2 * np.pi, 0.0)[0, 0], 24.573875828863905
        )
        self.assertAlmostEqual(
            beta_funcs[(("E", 1, 1), "real")](2 * np.pi, 0.0)[0, 0], 0.07516250767481286
        )
        self.assertAlmostEqual(
            beta_funcs[(("E", 1, 1), "imag")](2 * np.pi, 0.0)[0, 0], 8.2795729054555345
        )
        self.assertAlmostEqual(
            beta_funcs[(("E", 1, 2), "real")](2 * np.pi, 0.0)[0, 0], 0.1884476108781034
        )
        self.assertAlmostEqual(
            beta_funcs[(("E", 1, 2), "imag")](2 * np.pi, 0.0)[0, 0], 29.74494425189081
        )
        self.assertAlmostEqual(
            beta_funcs[(("E", 2, 1), "real")](2 * np.pi, 0.0)[0, 0], 0.10224849620607172
        )
        self.assertAlmostEqual(
            beta_funcs[(("E", 2, 1), "imag")](2 * np.pi, 0.0)[0, 0], 16.184787946722981
        )
        self.assertAlmostEqual(
            beta_funcs[(("E", 3, 1), "real")](2 * np.pi, 0.0)[0, 0], 0.1353140706821849
        )
        self.assertAlmostEqual(
            beta_funcs[(("E", 3, 1), "imag")](2 * np.pi, 0.0)[0, 0], 23.102611360449739
        )
        self.assertAlmostEqual(
            beta_funcs[(("E", 4, 1), "real")](2 * np.pi, 0.0)[0, 0], 0.1720347941605904
        )
        self.assertAlmostEqual(
            beta_funcs[(("E", 4, 1), "imag")](2 * np.pi, 0.0)[0, 0], 29.661836787913028
        )

    def test_eig_mat(self):
        size = self.params["core"]["size"]
        fill = self.params["core"]["fill"]
        clad = self.params["clad"]
        wg = Samples(size, fill, clad, self.params["modes"])
        h2 = -35513.94604091 - 6.53379717e5j
        w = 2 * np.pi / 100.0
        e1 = wg.fill(w)
        e2 = wg.clad(w) * 100
        eps = 1e-4
        a, b = wg.eig_mat(h2, w, "M", 0, e1, e2)
        a1, b1 = wg.eig_mat(h2 + eps, w, "M", 0, e1, e2)
        a2, b2 = wg.eig_mat(h2 - eps, w, "M", 0, e1, e2)
        da_dh2 = (a1 - a2) / (2 * eps)
        npt.assert_almost_equal(b, da_dh2)
        a, b = wg.eig_mat(h2, w, "E", 0, e1, e2)
        a1, b1 = wg.eig_mat(h2 + eps, w, "E", 0, e1, e2)
        a2, b2 = wg.eig_mat(h2 - eps, w, "E", 0, e1, e2)
        da_dh2 = (a1 - a2) / (2 * eps)
        npt.assert_almost_equal(b, da_dh2)
        a, b = wg.eig_mat(h2, w, "M", 2, e1, e2)
        a1, b1 = wg.eig_mat(h2 + eps, w, "M", 2, e1, e2)
        a2, b2 = wg.eig_mat(h2 - eps, w, "M", 2, e1, e2)
        da_dh2 = (a1 - a2) / (2 * eps)
        npt.assert_almost_equal(b, da_dh2)


if __name__ == "__main__":
    unittest.main()



import numpy as np
import numpy.testing as npt

from pymwm.coax.samples import Samples
from pymwm.utils import coax_utils


def test_eig_mat():
    size = 0.15
    fill = {"RI": 1.0}
    clad = {"book": "Au", "page": "Stewart-DLF", "bound_check": False}
    p = {
        "wl_max": 10.0,
        "wl_min": 1.0,
        "wl_imag": 50.0,
        "dw": 1.0 / 64,
        "num_n": 6,
        "num_m": 2,
    }
    size2 = 0.1
    wg = Samples(size, fill, clad, p, size2)
    h2 = -3500 + 6.3379717e5j
    w = 2 * np.pi
    e1 = wg.fill(w)
    e2 = wg.clad(w)
    eps = 1e-4
    # a = np.empty((4, 4), dtype=complex)
    # b = np.empty((4, 4), dtype=complex)
    # a1 = np.empty((4, 4), dtype=complex)
    # a2 = np.empty((4, 4), dtype=complex)
    # b1 = np.empty((4, 4), dtype=complex)
    # coax_utils.eig_mat_with_deriv(h2, w, "M", 0, e1, e2, size, size2, a, b)
    # coax_utils.eig_mat_with_deriv(h2 + eps, w, "M", 0, e1, e2, size, size2, a1, b1)
    # coax_utils.eig_mat_with_deriv(h2 - eps, w, "M", 0, e1, e2, size, size2, a2, b1)
    a, b = wg.eig_mat(h2, w, "M", 0, e1, e2)
    a1, _ = wg.eig_mat(h2 + eps, w, "M", 0, e1, e2)
    a2, _ = wg.eig_mat(h2 - eps, w, "M", 0, e1, e2)
    da_dh2_1 = (a1 - a2) / (2 * eps)
    a3, _ = wg.eig_mat(h2 + eps * 1j, w, "M", 0, e1, e2)
    a4, _ = wg.eig_mat(h2 - eps * 1j, w, "M", 0, e1, e2)
    da_dh2_2 = -1j * (a3 - a4) / (2 * eps)
    # print(f"{a}")
    # print(f"{b}")
    # print(f"{da_dh2_1}")
    # print(f"{da_dh2_2}")
    npt.assert_almost_equal(b, da_dh2_1)
    npt.assert_almost_equal(b, da_dh2_2)
    a, b = wg.eig_mat(h2, w, "E", 0, e1, e2)
    a1, _ = wg.eig_mat(h2 + eps, w, "E", 0, e1, e2)
    a2, _ = wg.eig_mat(h2 - eps, w, "E", 0, e1, e2)
    da_dh2_1 = (a1 - a2) / (2 * eps)
    a3, _ = wg.eig_mat(h2 + eps * 1j, w, "E", 0, e1, e2)
    a4, _ = wg.eig_mat(h2 - eps * 1j, w, "E", 0, e1, e2)
    da_dh2_2 = -1j * (a3 - a4) / (2 * eps)
    # print(f"{a}")
    # print(f"{b}")
    # print(f"{da_dh2_1}")
    # print(f"{da_dh2_2}")
    npt.assert_almost_equal(b, da_dh2_1)
    npt.assert_almost_equal(b, da_dh2_2)
    # coax_utils.eig_mat_with_deriv(h2, w, "M", 1, e1, e2, size, size2, a, b)
    # coax_utils.eig_mat_with_deriv(h2 + eps, w, "M", 1, e1, e2, size, size2, a1, b1)
    # coax_utils.eig_mat_with_deriv(h2 - eps, w, "M", 1, e1, e2, size, size2, a2, b1)
    a, b = wg.eig_mat(h2, w, "M", 1, e1, e2)
    a1, _ = wg.eig_mat(h2 + eps, w, "M", 1, e1, e2)
    a2, _ = wg.eig_mat(h2 - eps, w, "M", 1, e1, e2)
    da_dh2_1 = (a1 - a2) / (2 * eps)
    a3, _ = wg.eig_mat(h2 + eps * 1j, w, "M", 1, e1, e2)
    a4, _ = wg.eig_mat(h2 - eps * 1j, w, "M", 1, e1, e2)
    da_dh2_2 = -1j * (a3 - a4) / (2 * eps)
    # print(f"{a}")
    print(f"{b}")
    print(f"{da_dh2_1}")
    print(f"{da_dh2_2}")
    npt.assert_almost_equal(b, da_dh2_1)
    npt.assert_almost_equal(b, da_dh2_2)

    h2 = -3500 - 6.3379717e5j
    a, b = wg.eig_mat(h2, w, "M", 0, e1, e2)
    a1, _ = wg.eig_mat(h2 + eps, w, "M", 0, e1, e2)
    a2, _ = wg.eig_mat(h2 - eps, w, "M", 0, e1, e2)
    da_dh2_1 = (a1 - a2) / (2 * eps)
    a3, _ = wg.eig_mat(h2 + eps * 1j, w, "M", 0, e1, e2)
    a4, _ = wg.eig_mat(h2 - eps * 1j, w, "M", 0, e1, e2)
    da_dh2_2 = -1j * (a3 - a4) / (2 * eps)
    # print(f"{a}")
    # print(f"{b}")
    # print(f"{da_dh2_1}")
    # print(f"{da_dh2_2}")
    npt.assert_almost_equal(b, da_dh2_1)
    npt.assert_almost_equal(b, da_dh2_2)
    a, b = wg.eig_mat(h2, w, "E", 0, e1, e2)
    a1, _ = wg.eig_mat(h2 + eps, w, "E", 0, e1, e2)
    a2, _ = wg.eig_mat(h2 - eps, w, "E", 0, e1, e2)
    da_dh2_1 = (a1 - a2) / (2 * eps)
    a3, _ = wg.eig_mat(h2 + eps * 1j, w, "E", 0, e1, e2)
    a4, _ = wg.eig_mat(h2 - eps * 1j, w, "E", 0, e1, e2)
    da_dh2_2 = -1j * (a3 - a4) / (2 * eps)
    # print(f"{a}")
    # print(f"{b}")
    # print(f"{da_dh2_1}")
    # print(f"{da_dh2_2}")
    npt.assert_almost_equal(b, da_dh2_1)
    npt.assert_almost_equal(b, da_dh2_2)
    a, b = wg.eig_mat(h2, w, "M", 1, e1, e2)
    a1, _ = wg.eig_mat(h2 + eps, w, "M", 1, e1, e2)
    a2, _ = wg.eig_mat(h2 - eps, w, "M", 1, e1, e2)
    da_dh2_1 = (a1 - a2) / (2 * eps)
    a3, _ = wg.eig_mat(h2 + eps * 1j, w, "M", 1, e1, e2)
    a4, _ = wg.eig_mat(h2 - eps * 1j, w, "M", 1, e1, e2)
    da_dh2_2 = -1j * (a3 - a4) / (2 * eps)
    # print(f"{a}")
    print(f"{b}")
    print(f"{da_dh2_1}")
    print(f"{da_dh2_2}")
    npt.assert_almost_equal(b, da_dh2_1)
    npt.assert_almost_equal(b, da_dh2_2)


def test_eig_eq():
    size = 0.15
    fill = {"RI": 1.0}
    clad = {"book": "Au", "page": "Stewart-DLF", "bound_check": False}
    p = {
        "wl_max": 10.0,
        "wl_min": 1.0,
        "wl_imag": 50.0,
        "dw": 1.0 / 64,
        "num_n": 6,
        "num_m": 2,
    }
    size2 = 0.1
    wg = Samples(size, fill, clad, p, size2)
    h2 = -350 + 60j
    w = 2 * np.pi
    e1 = wg.fill(w)
    e2 = wg.clad(w)
    eps = 1e-4
    for pol in ["M", "E"]:
        for n in range(6):
            a, b = wg.eig_mat(h2, w, pol, n, e1, e2)
            a1, _ = wg.eig_mat(h2 + eps, w, pol, n, e1, e2)
            a2, _ = wg.eig_mat(h2 - eps, w, pol, n, e1, e2)
            if n == 0:
                if pol == "E":
                    a = a[:2, :2]
                    a1 = a1[:2, :2]
                    a2 = a2[:2, :2]
                else:
                    a = a[2:, 2:]
                    a1 = a1[2:, 2:]
                    a2 = a2[2:, 2:]
            f1, fp1 = wg.eig_eq(np.array([h2.real, h2.imag]), w, pol, n, e1, e2, np.array([]))
            f2 = np.linalg.det(a)
            fp2 = (np.linalg.det(a1) - np.linalg.det(a2)) / (2 * eps)
            npt.assert_array_almost_equal(f1, [f2.real, f2.imag])
            npt.assert_array_almost_equal(fp1, np.array(
                [[fp2.real, fp2.imag], [-fp2.imag, fp2.real]]))


def test_eig_eq_cython():
    size = 0.15
    fill = {"RI": 1.0}
    clad = {"book": "Au", "page": "Stewart-DLF", "bound_check": False}
    p = {
        "wl_max": 10.0,
        "wl_min": 1.0,
        "wl_imag": 50.0,
        "dw": 1.0 / 64,
        "num_n": 6,
        "num_m": 2,
    }
    size2 = 0.1
    wg = Samples(size, fill, clad, p, size2)
    h2 = 300 + 6j
    w = 2 * np.pi / 1.0
    e1 = wg.fill(w)
    e2 = wg.clad(w)
    eps = 1e-4
    for pol in ["M", "E"]:
        for n in range(6):
            a, b = wg.eig_mat(h2, w, pol, n, e1, e2)
            a1, _ = wg.eig_mat(h2 + eps, w, pol, n, e1, e2)
            a2, _ = wg.eig_mat(h2 - eps, w, pol, n, e1, e2)
            if n == 0:
                if pol == "E":
                    a = a[:2, :2]
                    a1 = a1[:2, :2]
                    a2 = a2[:2, :2]
                else:
                    a = a[2:, 2:]
                    a1 = a1[2:, 2:]
                    a2 = a2[2:, 2:]
            f1, fp1 = coax_utils.eig_eq_with_jac(
                np.array([h2.real, h2.imag]), w, pol, n, e1, e2, size, size2, np.array([], dtype=complex)
            )
            f2 = np.linalg.det(a)
            fp2 = (np.linalg.det(a1) - np.linalg.det(a2)) / (2 * eps)
            npt.assert_array_almost_equal(f1, [f2.real, f2.imag])
            npt.assert_array_almost_equal(fp1, np.array(
                [[fp2.real, fp2.imag], [-fp2.imag, fp2.real]]))



from __future__ import annotations

import numpy as np
import numpy.testing as npt
import scipy.special as ssp

from pymwm.utils.bessel_utils import yv_yvp_cython


def test_yv_yvp():
    for n in range(4):
        z = np.random.random() + 1j * np.random.random()
        y1, yp1 = yv_yvp_cython(n, z)
        y2 = ssp.yv(n, z)
        yp2 = ssp.yvp(n, z)
        npt.assert_almost_equal(y1, y2)
        npt.assert_almost_equal(yp1, yp2)



import unittest

import numpy as np
import numpy.testing as npt

import pymwm


class TestCylinderCoefs(unittest.TestCase):
    def setUp(self):
        self.params = {
            "core": {"shape": "cylinder", "size": 0.15, "fill": {"RI": 1.0}},
            "clad": {"book": "Au", "page": "Stewart-DLF"},
            "bounds": {"wl_max": 5.0, "wl_min": 1.0, "wl_imag": 100.0},
            "modes": {
                "wl_max": 5.0,
                "wl_min": 1.0,
                "wl_imag": 50.0,
                "num_n": 6,
                "num_m": 2,
                "ls": ["h", "v"],
            },
        }
        self.pec = {"PEC": True}

    def test_props(self):
        params = self.params.copy()
        wg = pymwm.create(params)
        wr = 2.0 * np.pi
        wi = -0.002
        w = wr + wi * 1j
        hs1, us1, vs1, jus1, jpus1, kvs1, kpvs1, As1, Bs1, Ys1 = wg.props_numpy(w)
        hs2, us2, vs2, jus2, jpus2, kvs2, kpvs2, As2, Bs2, Ys2 = wg.props(w)
        print(As1, As2)
        npt.assert_allclose(hs1, hs2)
        npt.assert_allclose(us1, us2)
        npt.assert_allclose(vs1, vs2)
        npt.assert_allclose(jus1, jus2)
        npt.assert_allclose(jpus1, jpus2)
        npt.assert_allclose(kvs1, kvs2)
        npt.assert_allclose(kpvs1, kpvs2)
        npt.assert_allclose(As1, As2)
        npt.assert_allclose(Bs1, Bs2)
        npt.assert_allclose(Ys1, Ys2)

        params["clad"] = self.pec
        wg = pymwm.create(params)
        # print(type(wg.clad))
        hs1, us1, vs1, jus1, jpus1, kvs1, kpvs1, As1, Bs1, Ys1 = wg.props_numpy(w)
        hs2, us2, vs2, jus2, jpus2, kvs2, kpvs2, As2, Bs2, Ys2 = wg.props(w)
        print(As1, As2)
        npt.assert_allclose(hs1, hs2)
        npt.assert_allclose(us1, us2)
        npt.assert_allclose(vs1, vs2)
        npt.assert_allclose(jus1, jus2)
        npt.assert_allclose(jpus1, jpus2)
        npt.assert_allclose(kvs1, kvs2)
        npt.assert_allclose(kpvs1, kpvs2)
        npt.assert_allclose(As1, As2)
        npt.assert_allclose(Bs1, Bs2)
        npt.assert_allclose(Ys1, Ys2)

    def test_norm(self):
        params = self.params.copy()
        wg = pymwm.create(params)
        wr = 2.0 * np.pi
        wi = -0.002
        w = complex(wr, wi)
        hs = np.array([wg.beta(w, alpha) for alpha in wg.alpha_all])
        As, Bs = wg.coefs(hs, w)
        for h, a, b, s, n, m in zip(hs, As, Bs, wg.s_all, wg.n_all, wg.m_all):
            pol = "E" if s == 0 else "M"
            norm = wg.norm(w, h, (pol, n, m), a, b)
            self.assertAlmostEqual(norm, 1.0)

        params["clad"] = self.pec
        wg = pymwm.create(params)
        hs = np.array([wg.beta(w, alpha) for alpha in wg.alpha_all])
        As, Bs = wg.coefs(hs, w)
        for h, a, b, s, n, m in zip(hs, As, Bs, wg.s_all, wg.n_all, wg.m_all):
            pol = "E" if s == 0 else "M"
            norm = wg.norm(w, h, (pol, n, m), a, b)
            self.assertAlmostEqual(norm, 1.0)


if __name__ == "__main__":
    unittest.main()



from __future__ import annotations

import numpy as np

def det_cython(a: np.ndarray) -> complex: ...
def solve_cython(a: np.ndarray, b: np.ndarray) -> tuple[complex, complex, complex]: ...
def det4_cython(a: np.ndarray) -> complex: ...
def deriv_det4_cython(a: np.ndarry, b: np.ndarray) -> complex: ...
def deriv_det2_cython(a: np.ndarry, b: np.ndarray) -> complex: ...



import unittest

from pymwm.waveguide import Database


class Load(unittest.TestCase):
    def setUp(self):
        key = dict(
            (
                ("shape", "cylinder"),
                ("size", 0.5),
                ("size2", 0.0),
                ("core", {"RI", 1.0}),
                ("clad", {"book": "Au", "page": "Stewart-DLF"}),
                ("wl_max", 1.0),
                ("wl_min", 0.4),
                ("wl_imag", 5.0),
                ("dw", 1 / 64),
                ("num_n", 6),
                ("num_m", 7),
                ("num_all", 30),
                ("im_factor", 1.0),
            )
        )
        self.database = Database(key)

    def test_load(self):
        with self.assertRaises(IndexError):
            self.database.load()


if __name__ == "__main__":
    unittest.main()



import numpy as np
import numpy.testing as npt

from pymwm.cylinder.samples import Samples
from pymwm.utils import cylinder_utils


def test_eiq_eq():
    size = 0.15
    fill = {"RI": 1.0}
    clad = {"book": "Au", "page": "Stewart-DLF", "bound_check": False}
    p = {
        "wl_max": 10.0,
        "wl_min": 1.0,
        "wl_imag": 50.0,
        "dw": 1.0 / 64,
        "num_n": 6,
        "num_m": 2,
    }
    roots = np.array([-1550 + 20j, -500 + 1.5j], dtype=complex)
    wg = Samples(size, fill, clad, p)
    w = 2 * np.pi / 100.0
    e1 = wg.fill(w)
    e2 = wg.clad(w) * 1000
    h2vec = np.array([-35513.94604091, -6.53379717e5])
    f1, fp1 = wg.eig_eq(h2vec, w, "E", 0, e1, e2, roots)
    f2, fp2 = cylinder_utils.eig_eq_cython(h2vec, w, "E", 0, e1, e2, size, roots)
    npt.assert_almost_equal(f1, f2)
    npt.assert_almost_equal(fp1, fp2)
    f1, fp1 = wg.eig_eq(h2vec, w, "M", 0, e1, e2, roots)
    f2, fp2 = cylinder_utils.eig_eq_cython(h2vec, w, "M", 0, e1, e2, size, roots)
    npt.assert_almost_equal(f1, f2)
    npt.assert_almost_equal(fp1, fp2)
    f1, fp1 = wg.eig_eq(h2vec, w, "M", 1, e1, e2, roots)
    f2, fp2 = cylinder_utils.eig_eq_cython(h2vec, w, "M", 1, e1, e2, size, roots)
    print(f1, fp1)
    npt.assert_almost_equal(f1, f2)
    npt.assert_almost_equal(fp1, fp2)


def test_eiq_eq_derivative():
    size = 0.15
    fill = {"RI": 1.0}
    clad = {"book": "Au", "page": "Stewart-DLF", "bound_check": False}
    p = {
        "wl_max": 10.0,
        "wl_min": 1.0,
        "wl_imag": 50.0,
        "dw": 1.0 / 64,
        "num_n": 6,
        "num_m": 2,
    }
    roots = np.array([-1550 + 20j, -500 + 1.5j], dtype=complex)
    wg = Samples(size, fill, clad, p)
    w = 2 * np.pi / 100.0
    e1 = wg.fill(w)
    e2 = wg.clad(w) * 1000
    h2vec = np.array([-35513.94604091, -6.53379717e5])
    eps = 1e-4
    f, fp = cylinder_utils.eig_eq_cython(h2vec, w, "E", 0, e1, e2, size, roots)
    f1, _ = cylinder_utils.eig_eq_cython(
        h2vec + np.array([eps, 0]), w, "E", 0, e1, e2, size, roots
    )
    f2, _ = cylinder_utils.eig_eq_cython(
        h2vec - np.array([eps, 0]), w, "E", 0, e1, e2, size, roots
    )
    f1 = f1[0] + f1[1] * 1j
    f2 = f2[0] + f2[1] * 1j
    fp1 = fp[0, 0] + fp[1, 0] * 1j
    fp2 = (f1 - f2) / (2 * eps)
    print(fp1, fp2)
    npt.assert_almost_equal(fp1, fp2)



import numpy as np
import numpy.testing as npt

from pymwm.utils import eig_mat_utils


def det_func(a0, z):
    a = a0 + np.array([[z, 0, 0, z], [0, z, 0, 0], [0, 0, z, 0], [-1, 0, z, 0]])
    b = np.array(
        [[1, 0, 0, 1], [0, 1, 0, 0], [0, 0, 1, 0], [0, 0, 1, 0]], dtype=complex
    )
    return np.array(a), np.array(b)


def deriv_det_func(a0, z):
    return (
        np.linalg.det(det_func(a0, z + 1e-4)[0])
        - np.linalg.det(det_func(a0, z - 1e-4)[0])
    ) / 2e-4


def test_solve_cython():
    a = np.random.random((3, 3)) + np.random.random((3, 3)) * 1j
    b = np.random.random(3) + np.random.random(3) * 1j
    val1 = np.linalg.solve(a, b)
    val2 = eig_mat_utils.solve_cython(a, b)
    print(val1, val2)
    npt.assert_almost_equal(val1, val2)


def test_det4():
    a = np.cos(3 * (np.random.random((4, 4)) + np.random.random((4, 4)) * 1j))
    val1 = np.linalg.det(a)
    val2 = eig_mat_utils.det4_cython(a)
    print(val1, val2)
    npt.assert_almost_equal(val1, val2)


def test_deriv_det4():
    from scipy.sparse.linalg import eigs

    a0 = np.random.random((4, 4)) + np.random.random((4, 4)) * 1j
    z = np.random.random() + np.random.random() * 1j
    a, b = det_func(a0, z)
    a1, _ = det_func(a0, z + 1e-4)
    a2, _ = det_func(a0, z - 1e-4)
    da_dz = (a1 - a2) / 2e-4
    val1 = eig_mat_utils.deriv_det4_cython(a, b)
    val2 = deriv_det_func(a0, z)
    print(f"{a}")
    print(f"{b}")
    print(f"{da_dz}")
    print(f"{np.linalg.det(a)}")
    print(f"{eigs(a, k=1, which = 'SM', return_eigenvectors=False)}")
    print(f"{np.linalg.eigvals(a)}")
    s, val = np.linalg.slogdet(a)
    print(f"{s * np.exp(val)}")
    print(f"{eig_mat_utils.det4_cython(a)}")
    print(f"{val1}")
    print(f"{val2}")
    npt.assert_allclose(b, da_dz)
    npt.assert_almost_equal(val1, val2)


def test_deriv_det2():
    a0 = np.random.random((2, 2)) + np.random.random((2, 2)) * 1j
    z = np.random.random() + np.random.random() * 1j
    a = a0 + np.array([[z, 0], [0, z]], dtype=complex)
    b = np.array([[1, 0], [0, 1]], dtype=complex)
    a1 = a0 + np.array([[z + 1e-4, 0], [0, z + 1e-4]], dtype=complex)
    a2 = a0 + np.array([[z - 1e-4, 0], [0, z - 1e-4]], dtype=complex)
    det_a1 = a1[0, 0] * a1[1, 1] - a1[1, 0] * a1[0, 1]
    det_a2 = a2[0, 0] * a2[1, 1] - a2[1, 0] * a2[0, 1]
    print(a1, a2)
    print(det_a1, det_a2)
    val1 = eig_mat_utils.deriv_det2_cython(a, b)
    val2 = (det_a1 - det_a2) / 2e-4
    print(val1, val2)
    npt.assert_allclose(val1, val2)



from __future__ import annotations

import numpy as np

def coefs_cython(
    hole: object, hs: np.ndarray, w: complex
) -> tuple[np.ndarray, np.ndarray]: ...
def ABY_cython(
    w: complex,
    r: float,
    s_all: np.ndarray,
    n_all: np.ndarray,
    hs: np.ndarray,
    e1: complex,
    e2: complex,
) -> tuple[np.ndarray, np.ndarray, np.ndarray]: ...
def uvABY_cython(
    w: complex,
    r: float,
    s_all: np.ndarray,
    n_all: np.ndarray,
    hs: np.ndarray,
    e1: complex,
    e2: complex,
) -> tuple[np.ndaray, np.ndarray, np.ndarray, np.ndarray, np.ndarray]: ...
def eig_eq_cython(
    h2: complex, w: complex, pol: str, n: int, e1: complex, e2: complex, r: float
) -> complex: ...
def func_cython(
    h2vec: np.ndarray,
    w: complex,
    pol: str,
    n: int,
    e1: complex,
    e2: complex,
    r: float,
    roots: np.ndarray,
) -> tuple[float, float]: ...



from __future__ import annotations

def f_fp_fpp_cython(
    x: float, r_ratio: float, n: int, pol: str
) -> tuple[float, float, float]: ...
def f_fprime_cython(
    x: float, r_ratio: float, n: int, pol: str
) -> tuple[float, float]: ...
def f_cython(x: float, r_ratio: float, n: int, pol: str) -> float: ...
def fprime_cython(x: float, r_ratio: float, n: int, pol: str) -> float: ...



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest

import numpy as np
import numpy.testing as npt

import pymwm


class TestSlitCoefs(unittest.TestCase):
    def setUp(self):
        self.params = {
            "core": {"shape": "slit", "size": 0.3, "fill": {"RI": 1.333}},
            "clad": {"book": "Au", "page": "Stewart-DLF"},
            "bounds": {"wl_max": 2.0, "wl_min": 1.0, "wl_imag": 50.0},
            "modes": {
                "wl_max": 2.5,
                "wl_min": 1.0,
                "wl_imag": 50.0,
                "num_n": 6,
                "num_m": 1,
                "ls": ["h", "v"],
            },
        }
        self.pec = {"PEC": True}

    def test_coefs(self):
        params = self.params.copy()
        wg = pymwm.create(params)
        wr = 2.0 * np.pi
        wi = -0.002
        w = wr + wi * 1j
        alpha_all = wg.alpha_all
        hs = np.array([wg.beta(w, alpha) for alpha in alpha_all])
        As1, Bs1 = wg.coefs_numpy(hs, w)
        As2, Bs2 = wg.coefs(hs, w)
        print(As1, As2)
        npt.assert_allclose(As1, As2)
        npt.assert_allclose(Bs1, Bs2)

        params["clad"] = self.pec
        wg = pymwm.create(params)
        alpha_all = wg.alpha_all
        hs = np.array([wg.beta(w, alpha) for alpha in alpha_all])
        As1, Bs1 = wg.coefs_numpy(hs, w)
        As2, Bs2 = wg.coefs(hs, w)
        print(As1, As2)
        npt.assert_allclose(As1, As2)
        npt.assert_allclose(Bs1, Bs2)

    def test_ABY(self):
        params = self.params.copy()
        wg = pymwm.create(params)
        wr = 2.0 * np.pi
        wi = -0.002
        w = wr + wi * 1j
        hs = np.array([wg.beta(w, alpha) for alpha in wg.alpha_all])
        As1, Bs1 = wg.coefs_numpy(hs, w)
        Y1 = wg.Ys(w, hs, As1, Bs1)
        As2, Bs2, Y2 = wg.ABY(w, hs)
        npt.assert_allclose(As1, As2)
        npt.assert_allclose(Bs1, Bs2)
        print(wg.alpha_all)
        print(Y1, Y2)
        npt.assert_allclose(Y1, Y2)

        params["clad"] = self.pec
        wg = pymwm.create(params)
        hs = np.array([wg.beta(w, alpha) for alpha in wg.alpha_all])
        As1, Bs1 = wg.coefs_numpy(hs, w)
        Y1 = wg.Ys(w, hs, As1, Bs1)
        As2, Bs2, Y2 = wg.ABY(w, hs)
        npt.assert_allclose(As1, As2)
        npt.assert_allclose(Bs1, Bs2)
        npt.assert_allclose(Y1, Y2)

    def test_hABY(self):
        params = self.params.copy()
        wg = pymwm.create(params)
        wr = 2.0 * np.pi
        wi = -0.002
        w = wr + wi * 1j
        hs1 = np.array([wg.beta(w, alpha) for alpha in wg.alpha_all])
        As1, Bs1 = wg.coefs_numpy(hs1, w)
        Y1 = wg.Ys(w, hs1, As1, Bs1)
        hs2, As2, Bs2, Y2 = wg.hABY(w)
        npt.assert_allclose(As1, As2)
        npt.assert_allclose(Bs1, Bs2)
        npt.assert_allclose(Y1, Y2)

        params["clad"] = self.pec
        wg = pymwm.create(params)
        hs1 = np.array([wg.beta(w, alpha) for alpha in wg.alpha_all])
        As1, Bs1 = wg.coefs_numpy(hs1, w)
        Y1 = wg.Ys(w, hs1, As1, Bs1)
        hs2, As2, Bs2, Y2 = wg.hABY(w)
        npt.assert_allclose(As1, As2)
        npt.assert_allclose(Bs1, Bs2)
        npt.assert_allclose(Y1, Y2)

    def test_norm(self):
        params = self.params.copy()
        wg = pymwm.create(params)
        wr = 2.0 * np.pi
        wi = -0.002
        w = complex(wr, wi)
        hs = np.array([wg.beta(w, alpha) for alpha in wg.alpha_all])
        As, Bs = wg.coefs_numpy(hs, w)
        for h, a, b, s, n, m in zip(hs, As, Bs, wg.s_all, wg.n_all, wg.m_all):
            pol = "E" if s == 0 else "M"
            norm = wg.norm(w, h, (pol, n, m), a, b)
            self.assertAlmostEqual(norm, 1.0)

        params["clad"] = self.pec
        wg = pymwm.create(params)
        hs = np.array([wg.beta(w, alpha) for alpha in wg.alpha_all])
        As, Bs = wg.coefs_numpy(hs, w)
        for h, a, b, s, n, m in zip(hs, As, Bs, wg.s_all, wg.n_all, wg.m_all):
            pol = "E" if s == 0 else "M"
            norm = wg.norm(w, h, (pol, n, m), a, b)
            self.assertAlmostEqual(norm, 1.0)


if __name__ == "__main__":
    unittest.main()



from __future__ import annotations

import numpy as np
import numpy.testing as npt
from scipy.integrate import quad

import pymwm

params: dict = {
    "core": {"shape": "coax", "r": 0.15, "ri": 0.1, "fill": {"RI": 1.0}},
    "clad": {"book": "Au", "page": "Stewart-DLF", "bound_check": False},
    "bounds": {"wl_max": 5.0, "wl_min": 1.0, "wl_imag": 50.0},
    "modes": {"wl_max": 20.0, "wl_min": 1.0, "wl_imag": 50.0, "num_n": 6, "num_m": 2},
}


def test_norm():
    wg = pymwm.create(params)
    w = 2 * np.pi / 1.0
    alpha = ("E", 1, 1)
    en = 1 if alpha[1] == 0 else 2
    h = wg.beta(w, alpha)
    coef = np.random.random(8) + 1j * np.random.random(8)
    # coef = np.arange(1, 9) * 0.1 + np.arange(1, 9) * 0.01j
    I1 = wg.norm(h, w, alpha, coef)

    def rfunc(r):
        er, ep, _ = wg.e_field_r_dep(r, w, alpha, h, coef)
        return 2 * np.pi / en * r * (er ** 2 + ep ** 2).real

    def ifunc(r):
        er, ep, _ = wg.e_field_r_dep(r, w, alpha, h, coef)
        return 2 * np.pi / en * r * (er ** 2 + ep ** 2).imag

    rmin = 0.0
    rmax = 6
    I2 = np.sqrt(
        quad(rfunc, rmin, rmax, epsabs=1e-10, epsrel=1e-10)[0]
        + 1j * quad(ifunc, rmin, rmax, epsabs=1e-10, epsrel=1e-10)[0]
    )
    print(I1, I2)
    print(rfunc(rmax), ifunc(rmax))
    npt.assert_almost_equal(I1, I2)


def test_Y():
    wg = pymwm.create(params)
    w = 2 * np.pi / 1.0
    alpha = ("E", 1, 1)
    en = 1 if alpha[1] == 0 else 2
    h = wg.beta(w, alpha)
    coef = np.random.random(8) + 1j * np.random.random(8)
    Y1 = wg.Y(w, h, alpha, coef)

    def rfunc(r):
        er, ep, _, hr, hp, _ = wg.field_r_dep(r, w, alpha, h, coef)
        return 2 * np.pi / en * r * (hp * er - hr * ep).real

    def ifunc(r):
        er, ep, _, hr, hp, _ = wg.field_r_dep(r, w, alpha, h, coef)
        return 2 * np.pi / en * r * (hp * er - hr * ep).imag

    rmin = 0.0
    rmax = 6
    Y2 = (
        quad(rfunc, rmin, rmax, epsabs=1e-10, epsrel=1e-10)[0]
        + 1j * quad(ifunc, rmin, rmax, epsabs=1e-10, epsrel=1e-10)[0]
    )
    print(Y1, Y2)
    print(rfunc(rmax), ifunc(rmax))
    npt.assert_almost_equal(Y1, Y2)

    alpha = ("M", 0, 1)
    en = 1 if alpha[1] == 0 else 2
    h = wg.beta(w, alpha)
    coef = np.random.random(8) + 1j * np.random.random(8)
    Y1 = wg.Y(w, h, alpha, coef)
    Y2 = (
        quad(rfunc, rmin, rmax, epsabs=1e-10, epsrel=1e-10)[0]
        + 1j * quad(ifunc, rmin, rmax, epsabs=1e-10, epsrel=1e-10)[0]
    )
    print(Y1, Y2)
    print(rfunc(rmax), ifunc(rmax))
    npt.assert_almost_equal(Y1, Y2)


def test_props():
    wg = pymwm.create(params)
    w = 2 * np.pi / 1.0
    (
        hs1,
        xs1,
        ys1,
        us1,
        vs1,
        jxs1,
        jpxs1,
        yxs1,
        ypxs1,
        iys1,
        ipys1,
        jus1,
        jpus1,
        yus1,
        ypus1,
        kvs1,
        kpvs1,
        A1s1,
        B1s1,
        A2s1,
        B2s1,
        C2s1,
        D2s1,
        A3s1,
        B3s1,
        Ys1,
    ) = wg.props_numpy(w)
    (
        hs2,
        xs2,
        ys2,
        us2,
        vs2,
        jxs2,
        jpxs2,
        yxs2,
        ypxs2,
        iys2,
        ipys2,
        jus2,
        jpus2,
        yus2,
        ypus2,
        kvs2,
        kpvs2,
        A1s2,
        B1s2,
        A2s2,
        B2s2,
        C2s2,
        D2s2,
        A3s2,
        B3s2,
        Ys2,
    ) = wg.props(w)
    npt.assert_array_equal(hs1, hs2)
    npt.assert_array_almost_equal(xs1, xs2)
    npt.assert_array_almost_equal(ys1, ys2)
    npt.assert_array_almost_equal(us1, us2)
    npt.assert_array_almost_equal(vs1, vs2)
    npt.assert_array_almost_equal(jxs1, jxs2)
    npt.assert_array_almost_equal(jpxs1, jpxs2)
    npt.assert_array_almost_equal(yxs1, yxs2)
    npt.assert_array_almost_equal(ypxs1, ypxs2)
    npt.assert_array_almost_equal(iys1, iys2)
    npt.assert_array_almost_equal(ipys1, ipys2)
    npt.assert_array_almost_equal(jus1, jus2)
    npt.assert_array_almost_equal(jpus1, jpus2)
    npt.assert_array_almost_equal(yus1, yus2)
    npt.assert_array_almost_equal(ypus1, ypus2)
    npt.assert_array_almost_equal(kvs1, kvs2)
    npt.assert_array_almost_equal(kpvs1, kpvs2)
    npt.assert_allclose(A1s1, A1s2)
    npt.assert_allclose(B1s1, B1s2)
    npt.assert_allclose(A2s1, A2s2)
    npt.assert_allclose(B2s1, B2s2)
    npt.assert_allclose(C2s1, C2s2)
    npt.assert_allclose(D2s1, D2s2)
    npt.assert_allclose(A3s1, A3s2)
    npt.assert_allclose(B3s1, B3s2)
    print(Ys1)
    print(Ys2)
    npt.assert_allclose(Ys1, Ys2)


def test_continuity():
    delta = 1e-14
    wg = pymwm.create(params)
    w = 2 * np.pi / 1.0
    for alpha in wg.alphas["h"]:
        h = wg.beta(w, alpha)
        coef = wg.coef(h, w, alpha)
        ex1, ey1, ez1, hx1, hy1, hz1 = wg.fields(0, 0.1 - delta, w, "h", alpha, h, coef)
        ex2, ey2, ez2, hx2, hy2, hz2 = wg.fields(0, 0.1 + delta, w, "h", alpha, h, coef)
        ex3, ey3, ez3, hx3, hy3, hz3 = wg.fields(
            0, 0.15 - delta, w, "h", alpha, h, coef
        )
        ex4, ey4, ez4, hx4, hy4, hz4 = wg.fields(
            0, 0.15 + delta, w, "h", alpha, h, coef
        )
        npt.assert_array_almost_equal([ex1, ez1, hx1, hz1], [ex2, ez2, hx2, hz2])
        npt.assert_array_almost_equal([ex3, ez3, hx3, hz3], [ex4, ez4, hx4, hz4])
    for alpha in wg.alphas["v"]:
        h = wg.beta(w, alpha)
        coef = wg.coef(h, w, alpha)
        ex1, ey1, ez1, hx1, hy1, hz1 = wg.fields(0.1 - delta, 0, w, "h", alpha, h, coef)
        ex2, ey2, ez2, hx2, hy2, hz2 = wg.fields(0.1 + delta, 0, w, "h", alpha, h, coef)
        ex3, ey3, ez3, hx3, hy3, hz3 = wg.fields(
            0.15 - delta, 0, w, "h", alpha, h, coef
        )
        ex4, ey4, ez4, hx4, hy4, hz4 = wg.fields(
            0.15 + delta, 0, w, "h", alpha, h, coef
        )
        npt.assert_array_almost_equal([ey1, ez1, hy1, hz1], [ey2, ez2, hy2, hz2])
        npt.assert_array_almost_equal([ey3, ez3, hy3, hz3], [ey4, ez4, hy4, hz4])



from __future__ import annotations

import numpy as np
import numpy.testing as npt
import scipy.special as ssp
from scipy.integrate import quad, romb

import pymwm

params: dict = {
    "core": {"shape": "coax", "r": 0.15, "ri": 0.1, "fill": {"RI": 1.0}},
    "clad": {"book": "Au", "page": "Stewart-DLF", "bound_check": False},
    "bounds": {"wl_max": 5.0, "wl_min": 1.0, "wl_imag": 50.0},
    "modes": {"wl_max": 20.0, "wl_min": 1.0, "wl_imag": 50.0, "num_n": 6, "num_m": 2},
}


def test_II():
    rmin = 0.1
    rmax = 0.2

    def F(r, n, a, b):
        if a == b:
            return (
                -(r ** 2)
                / 2
                * (
                    ssp.ivp(n, a * r) ** 2
                    - (1 + n ** 2 / (a * r) ** 2) * ssp.iv(n, a * r) ** 2
                )
            )
        else:
            return -(
                b * r * ssp.iv(n, a * r) * ssp.ivp(n, b * r)
                - a * r * ssp.ivp(n, a * r) * ssp.iv(n, b * r)
            ) / (a ** 2 - b ** 2)

    for n in range(3):
        for a in [1.0, 1.0 + 0.5j, 2.0 + 0.3j]:
            for b in [1.0, 1.0 + 0.5j, 2.0 + 0.3j]:

                def rfunc(r):
                    return (r * ssp.iv(n, a * r) * ssp.iv(n, b * r)).real

                def ifunc(r):
                    return (r * ssp.iv(n, a * r) * ssp.iv(n, b * r)).imag

                I1 = F(rmax, n, a, b) - F(rmin, n, a, b)
                I2 = quad(rfunc, rmin, rmax)[0] + 1j * quad(ifunc, rmin, rmax)[0]
                print(I1, I2)
                npt.assert_almost_equal(I1, I2)


def test_JJ():
    rmin = 0.1
    rmax = 0.2

    def F(r, n, a, b):
        if a == b:
            return (
                r ** 2
                / 2
                * (
                    ssp.jvp(n, a * r) ** 2
                    + (1 - n ** 2 / (a * r) ** 2) * ssp.jv(n, a * r) ** 2
                )
            )
        else:
            return (
                b * r * ssp.jv(n, a * r) * ssp.jvp(n, b * r)
                - a * r * ssp.jvp(n, a * r) * ssp.jv(n, b * r)
            ) / (a ** 2 - b ** 2)

    for n in range(3):
        for a in [1.0, 1.0 + 0.5j, 2.0 + 0.3j]:
            for b in [1.0, 1.0 + 0.5j, 2.0 + 0.3j]:
                print(n, a, b)

                def rfunc(r):
                    return (r * ssp.jv(n, a * r) * ssp.jv(n, b * r)).real

                def ifunc(r):
                    return (r * ssp.jv(n, a * r) * ssp.jv(n, b * r)).imag

                I1 = F(rmax, n, a, b) - F(rmin, n, a, b)
                I2 = quad(rfunc, rmin, rmax)[0] + 1j * quad(ifunc, rmin, rmax)[0]
                print(I1, I2)
                npt.assert_almost_equal(I1.real, I2.real)


def test_JY():
    rmin = 0.1
    rmax = 0.2

    def F(r, n, a, b):
        if a == b:
            return (
                r ** 2
                / 2
                * (
                    ssp.jvp(n, a * r) * ssp.yvp(n, a * r)
                    + (1 - n ** 2 / (a * r) ** 2) * ssp.jv(n, a * r) * ssp.yv(n, a * r)
                )
            )
        else:
            return (
                b * r * ssp.jv(n, a * r) * ssp.yvp(n, b * r)
                - a * r * ssp.jvp(n, a * r) * ssp.yv(n, b * r)
            ) / (a ** 2 - b ** 2)

    for n in range(3):
        for a in [1.0, 1.0 + 0.5j, 2.0 + 0.3j]:
            for b in [1.0, 1.0 + 0.5j, 2.0 + 0.3j]:
                print(n, a, b)

                def rfunc(r):
                    return (r * ssp.jv(n, a * r) * ssp.yv(n, b * r)).real

                def ifunc(r):
                    return (r * ssp.jv(n, a * r) * ssp.yv(n, b * r)).imag

                I1 = F(rmax, n, a, b) - F(rmin, n, a, b)
                I2 = quad(rfunc, rmin, rmax)[0] + 1j * quad(ifunc, rmin, rmax)[0]
                print(I1, I2)
                npt.assert_almost_equal(I1.real, I2.real)


def test_IJ():
    rmin = 0.1
    rmax = 0.2

    def F(r, n, a, b):
        return -(
            b * r * ssp.iv(n, a * r) * ssp.jvp(n, b * r)
            - a * r * ssp.ivp(n, a * r) * ssp.jv(n, b * r)
        ) / (a ** 2 + b ** 2)

    for n in range(3):
        for a in [1.0, 1.0 + 0.5j, 2.0 + 0.3j]:
            for b in [1.0, 1.0 + 0.5j, 2.0 + 0.3j]:
                print(n, a, b)

                def rfunc(r):
                    return (r * ssp.iv(n, a * r) * ssp.jv(n, b * r)).real

                def ifunc(r):
                    return (r * ssp.iv(n, a * r) * ssp.jv(n, b * r)).imag

                I1 = F(rmax, n, a, b) - F(rmin, n, a, b)
                I2 = quad(rfunc, rmin, rmax)[0] + 1j * quad(ifunc, rmin, rmax)[0]
                print(I1, I2)
                npt.assert_almost_equal(I1.real, I2.real)


def test_KJ():
    r0 = 0.1

    def F(r, n, a, b):
        return (
            b * r * ssp.kv(n, a * r) * ssp.jvp(n, b * r)
            - a * r * ssp.kvp(n, a * r) * ssp.jv(n, b * r)
        ) / (a ** 2 + b ** 2)

    for n in range(3):
        for a in [1.0, 1.0 + 0.5j, 2.0 + 0.3j]:
            for b in [1.0, 1.0 + 0.5j, 2.0 + 0.3j]:
                print(n, a, b)

                def rfunc(r):
                    return (r * ssp.kv(n, a * r) * ssp.jv(n, b * r)).real

                def ifunc(r):
                    return (r * ssp.kv(n, a * r) * ssp.jv(n, b * r)).imag

                I1 = F(r0, n, a, b)
                I2 = quad(rfunc, r0, 200)[0] + 1j * quad(ifunc, r0, 200)[0]
                print(I1, I2)
                npt.assert_almost_equal(I1.real, I2.real)


def test_KK():
    r0 = 0.1

    def F(r, n, a, b):
        if a == b:
            return (
                r ** 2
                / 2
                * (
                    ssp.kvp(n, a * r) ** 2
                    - (1 + n ** 2 / (a * r) ** 2) * ssp.kv(n, a * r) ** 2
                )
            )
        else:
            return (
                b * r * ssp.kv(n, a * r) * ssp.kvp(n, b * r)
                - a * r * ssp.kvp(n, a * r) * ssp.kv(n, b * r)
            ) / (a ** 2 - b ** 2)

    for n in range(3):
        for a in [1.0, 1.0 + 0.5j, 2.0 + 0.3j]:
            for b in [1.0, 1.0 + 0.5j, 2.0 + 0.3j]:

                def rfunc(r):
                    return (r * ssp.kv(n, a * r) * ssp.kv(n, b * r)).real

                def ifunc(r):
                    return (r * ssp.kv(n, a * r) * ssp.kv(n, b * r)).imag

                I1 = F(r0, n, a, b)
                I2 = quad(rfunc, r0, 200)[0] + 1j * quad(ifunc, r0, 200)[0]
                print(I1, I2)
                npt.assert_almost_equal(I1, I2)



import numpy as np
import numpy.testing as npt

from pymwm.cylinder.samples import Samples


def test_eig_mat():
    size = 0.15
    fill = {"RI": 1.0}
    clad = {"book": "Au", "page": "Stewart-DLF", "bound_check": False}
    p = {
        "wl_max": 10.0,
        "wl_min": 1.0,
        "wl_imag": 50.0,
        "dw": 1.0 / 64,
        "num_n": 6,
        "num_m": 2,
    }
    wg = Samples(size, fill, clad, p)
    h2 = -3500 + 6.3379717e5j
    w = 2 * np.pi / 100.0
    e1 = wg.fill(w)
    e2 = wg.clad(w) * 1000
    eps = 1e-4
    a, b = wg.eig_mat(h2, w, "M", 0, e1, e2)
    a1, _ = wg.eig_mat(h2 + eps, w, "M", 0, e1, e2)
    a2, _ = wg.eig_mat(h2 - eps, w, "M", 0, e1, e2)
    da_dh2_1 = (a1 - a2) / (2 * eps)
    a3, _ = wg.eig_mat(h2 + eps * 1j, w, "M", 0, e1, e2)
    a4, _ = wg.eig_mat(h2 - eps * 1j, w, "M", 0, e1, e2)
    da_dh2_2 = -1j * (a3 - a4) / (2 * eps)
    print(f"{a}")
    print(f"{b}")
    print(f"{da_dh2_1}")
    print(f"{da_dh2_2}")
    npt.assert_almost_equal(b, da_dh2_1)
    npt.assert_almost_equal(b, da_dh2_2)
    a, b = wg.eig_mat(h2, w, "E", 0, e1, e2)
    a1, _ = wg.eig_mat(h2 + eps, w, "E", 0, e1, e2)
    a2, _ = wg.eig_mat(h2 - eps, w, "E", 0, e1, e2)
    da_dh2_1 = (a1 - a2) / (2 * eps)
    a3, _ = wg.eig_mat(h2 + eps * 1j, w, "E", 0, e1, e2)
    a4, _ = wg.eig_mat(h2 - eps * 1j, w, "E", 0, e1, e2)
    da_dh2_2 = -1j * (a3 - a4) / (2 * eps)
    print(f"{a}")
    print(f"{b}")
    print(f"{da_dh2_1}")
    print(f"{da_dh2_2}")
    npt.assert_almost_equal(b, da_dh2_1)
    npt.assert_almost_equal(b, da_dh2_2)
    a, b = wg.eig_mat(h2, w, "M", 1, e1, e2)
    a1, _ = wg.eig_mat(h2 + eps, w, "M", 1, e1, e2)
    a2, _ = wg.eig_mat(h2 - eps, w, "M", 1, e1, e2)
    da_dh2_1 = (a1 - a2) / (2 * eps)
    a3, _ = wg.eig_mat(h2 + eps * 1j, w, "M", 1, e1, e2)
    a4, _ = wg.eig_mat(h2 - eps * 1j, w, "M", 1, e1, e2)
    da_dh2_2 = -1j * (a3 - a4) / (2 * eps)
    print(f"{a}")
    print(f"{b}")
    print(f"{da_dh2_1}")
    print(f"{da_dh2_2}")
    npt.assert_almost_equal(b, da_dh2_1)
    npt.assert_almost_equal(b, da_dh2_2)

    h2 = -3500 - 6.3379717e5j
    a, b = wg.eig_mat(h2, w, "M", 0, e1, e2)
    a1, _ = wg.eig_mat(h2 + eps, w, "M", 0, e1, e2)
    a2, _ = wg.eig_mat(h2 - eps, w, "M", 0, e1, e2)
    da_dh2_1 = (a1 - a2) / (2 * eps)
    a3, _ = wg.eig_mat(h2 + eps * 1j, w, "M", 0, e1, e2)
    a4, _ = wg.eig_mat(h2 - eps * 1j, w, "M", 0, e1, e2)
    da_dh2_2 = -1j * (a3 - a4) / (2 * eps)
    print(f"{a}")
    print(f"{b}")
    print(f"{da_dh2_1}")
    print(f"{da_dh2_2}")
    npt.assert_almost_equal(b, da_dh2_1)
    npt.assert_almost_equal(b, da_dh2_2)
    a, b = wg.eig_mat(h2, w, "E", 0, e1, e2)
    a1, _ = wg.eig_mat(h2 + eps, w, "E", 0, e1, e2)
    a2, _ = wg.eig_mat(h2 - eps, w, "E", 0, e1, e2)
    da_dh2_1 = (a1 - a2) / (2 * eps)
    a3, _ = wg.eig_mat(h2 + eps * 1j, w, "E", 0, e1, e2)
    a4, _ = wg.eig_mat(h2 - eps * 1j, w, "E", 0, e1, e2)
    da_dh2_2 = -1j * (a3 - a4) / (2 * eps)
    print(f"{a}")
    print(f"{b}")
    print(f"{da_dh2_1}")
    print(f"{da_dh2_2}")
    npt.assert_almost_equal(b, da_dh2_1)
    npt.assert_almost_equal(b, da_dh2_2)
    a, b = wg.eig_mat(h2, w, "M", 1, e1, e2)
    a1, _ = wg.eig_mat(h2 + eps, w, "M", 1, e1, e2)
    a2, _ = wg.eig_mat(h2 - eps, w, "M", 1, e1, e2)
    da_dh2_1 = (a1 - a2) / (2 * eps)
    a3, _ = wg.eig_mat(h2 + eps * 1j, w, "M", 1, e1, e2)
    a4, _ = wg.eig_mat(h2 - eps * 1j, w, "M", 1, e1, e2)
    da_dh2_2 = -1j * (a3 - a4) / (2 * eps)
    print(f"{a}")
    print(f"{b}")
    print(f"{da_dh2_1}")
    print(f"{da_dh2_2}")
    npt.assert_almost_equal(b, da_dh2_1)
    npt.assert_almost_equal(b, da_dh2_2)



import cmath

import numpy as np
import numpy.testing as npt

from pymwm.coax.samples import Samples
from pymwm.utils import coax_utils


def uo(h2: complex, w: complex, e2: complex, ri: float) -> complex:
    return (1 - 1j) * cmath.sqrt(0.5j * (h2 - e2 * w ** 2)) * ri


def vo(h2: complex, w: complex, e1: complex, ri: float) -> complex:
    return (1 + 1j) * cmath.sqrt(-0.5j * (e1 * w ** 2 - h2)) * ri


def xo(h2: complex, w: complex, e1: complex, r: float) -> complex:
    return (1 + 1j) * cmath.sqrt(-0.5j * (e1 * w ** 2 - h2)) * r


def yo(h2: complex, w: complex, e2: complex, r: float) -> complex:
    return (1 - 1j) * cmath.sqrt(0.5j * (h2 - e2 * w ** 2)) * r


def eig_eq_o(
    h2: complex,
    w: complex,
    pol: str,
    n: int,
    e1: complex,
    e2: complex,
    r: float,
    ri: float,
):
    w2 = w ** 2
    u = uo(h2, w, e2, ri)
    v = vo(h2, w, e1, ri)
    x = xo(h2, w, e1, r)
    y = yo(h2, w, e2, r)

    def F(f_name, z):
        sign = 1 if f_name == "iv" else -1
        func = eval("scipy.special." + f_name)
        f = func(n, z)
        fp = sign * func(n + 1, z) + n / z * f
        return f, fp / f

    jv, Fjv = F("jv", v)
    jx, Fjx = F("jv", x)
    yv, Fyv = F("yv", v)
    yx, Fyx = F("yv", x)
    iu, Fiu = F("iv", u)
    ky, Fky = F("kv", y)

    if n == 0:
        if pol == "M":
            val = (
                w2
                * (x * Fky + e1 / e2 * y * Fjx)
                / yx
                * w2
                * (v * Fiu + e1 / e2 * u * Fyv)
                - w2
                * (x * Fky + e1 / e2 * y * Fyx)
                * jv
                / jx
                * w2
                * (v * Fiu + e1 / e2 * u * Fjv)
                / yv
            )
            val /= w2 ** 2 * x * y / jx / yx / yv
        else:
            val = (x * Fky + y * Fjx) / yx * (v * Fiu + u * Fyv) - (
                v * Fiu + u * Fjv
            ) / yv * (x * Fky + y * Fyx) * jv / jx
            val /= x * y / jx / yx / yv
    else:
        nuv = n * (u / v + v / u)
        nxy = n * (x / y + y / x)

        a11 = (x * Fky + y * Fjx) / yx
        a12 = (x * Fky + y * Fyx) * jv / jx
        a13 = nxy / yx
        a14 = nxy * jv / jx

        a21 = (v * Fiu + u * Fjv) / yv
        a22 = v * Fiu + u * Fyv
        a23 = nuv / yv
        a24 = nuv

        a31 = h2 * nxy / e2 / yx
        a32 = h2 * nxy / e2 * jv / jx
        a33 = w2 * (x * Fky + e1 / e2 * y * Fjx) / yx
        a34 = w2 * (x * Fky + e1 / e2 * y * Fyx) * jv / jx

        a41 = h2 * nuv / e2 / yv
        a42 = h2 * nuv / e2
        a43 = w2 * (v * Fiu + e1 / e2 * u * Fjv) / yv
        a44 = w2 * (v * Fiu + e1 / e2 * u * Fyv)

        val = np.linalg.det(
            np.array(
                [
                    [a11, a12, a13, a14],
                    [a21, a22, a23, a24],
                    [a31, a32, a33, a34],
                    [a41, a42, a43, a44],
                ]
            )
        )
        val /= (w2 * x * y / jx / yx / yv) ** 2
    return val


def test_eiq_eq():
    size = 0.15
    size2 = 0.1
    fill = {"RI": 1.0}
    clad = {"book": "Au", "page": "Stewart-DLF", "bound_check": False}
    params = {
        "wl_max": 5.0,
        "wl_min": 1.0,
        "wl_imag": 50.0,
        "dw": 1.0 / 64,
        "num_n": 6,
        "num_m": 2,
    }
    roots = np.array([-1550 + 20j, -500 + 1.5j], dtype=complex)
    sm = Samples(size, fill, clad, params, size2)
    for pol in ("M", "E"):
        for n in range(5):
            w = sm.ws[0] - 0.1j
            h2 = 5 * w ** 2
            h2vec = np.array([h2.real, h2.imag])
            e1 = sm.fill(w)
            e2 = sm.clad(w)
            f1, fp1 = sm.eig_eq(h2vec, w, pol, n, e1, e2, roots)
            f2, fp2 = coax_utils.eig_eq_with_jac(
                h2vec, w, pol, n, e1, e2, size, size2, roots
            )
            print(pol, n)
            print(f1)
            print(f2)
            print(fp1)
            print(fp2)
            npt.assert_almost_equal(f1, f2)
            npt.assert_almost_equal(fp1, fp2)


def test_eiq_eq_for_min():
    size = 0.15
    size2 = 0.1
    fill = {"RI": 1.0}
    clad = {"book": "Au", "page": "Stewart-DLF", "bound_check": False}
    params = {
        "wl_max": 5.0,
        "wl_min": 1.0,
        "wl_imag": 50.0,
        "dw": 1.0 / 64,
        "num_n": 6,
        "num_m": 2,
    }
    roots = np.array([-1550 + 20j, -500 + 1.5j], dtype=complex)
    sm = Samples(size, fill, clad, params, size2)
    # for pol in ("M", "E"):
    #     for n in range(5):
    w = sm.ws[0] - 0.1j
    pol = "M"
    n = 1
    h2 = 5 * w ** 2
    h2vec = np.array([h2.real, h2.imag])
    e1 = sm.fill(w)
    e2 = sm.clad(w)
    f, fp = coax_utils.eig_eq_for_min_with_jac(
        h2vec, w, pol, n, e1, e2, size, size2, roots
    )
    f1, _ = coax_utils.eig_eq_for_min_with_jac(
        h2vec + np.array([1e-4, 0]), w, pol, n, e1, e2, size, size2, roots
    )
    f2, _ = coax_utils.eig_eq_for_min_with_jac(
        h2vec - np.array([1e-4, 0]), w, pol, n, e1, e2, size, size2, roots
    )
    f3, _ = coax_utils.eig_eq_for_min_with_jac(
        h2vec + np.array([0, 1e-4]), w, pol, n, e1, e2, size, size2, roots
    )
    f4, _ = coax_utils.eig_eq_for_min_with_jac(
        h2vec - np.array([0, 1e-4]), w, pol, n, e1, e2, size, size2, roots
    )
    fpr = (f1 - f2) / 2e-4
    fpi = (f3 - f4) / 2e-4
    print(f"{f}")
    print(f"{fp}")
    print(f"{fpr}")
    print(f"{fpi}")
    npt.assert_almost_equal(fp, np.array([fpr, fpi]))


def run1(cy):
    size = 0.15
    size2 = 0.1
    fill = {"RI": 1.0}
    clad = {"book": "Au", "page": "Stewart-DLF", "bound_check": False}
    params = {
        "wl_max": 5.0,
        "wl_min": 1.0,
        "wl_imag": 50.0,
        "dw": 1.0 / 64,
        "num_n": 6,
        "num_m": 2,
    }
    roots = np.array([-1550 + 20j, -500 + 1.5j], dtype=complex)
    sm = Samples(size, fill, clad, params, size2)
    w = sm.ws[0] - 0.1j
    h2 = 5 * w ** 2
    h2vec = np.array([h2.real, h2.imag])
    e1 = sm.fill(w)
    e2 = sm.clad(w)
    for pol in ("M", "E"):
        for n in range(5):
            if cy:
                [
                    coax_utils.eig_eq_cython(
                        h2vec, w, pol, n, e1, e2, size, size2, roots
                    )
                    for i in range(10000)
                ]
            else:
                [sm.eig_eq(h2vec, w, pol, n, e1, e2, roots) for i in range(10000)]



from __future__ import annotations

import numpy as np

# def uvABY_cython(
#     w: complex,
#     r: float,
#     s_all: np.ndarray,
#     n_all: np.ndarray,
#     m_all: np.ndarray,
#     hs: np.ndarray,
#     e1: complex,
#     e2: complex,
#     u_pec: np.ndarray,
#     jnu_pec: np.ndarray,
#     jnpy_pec: np.ndarray,
# ) -> tuple[np.ndaray, np.ndarray, np.ndarray, np.ndarray, np.ndarray]: ...
def props_cython(
    w: complex,
    r: float,
    s_all: np.ndarray,
    n_all: np.ndarray,
    m_all: np.ndarray,
    hs: np.ndarray,
    e1: complex,
    e2: complex,
    u_pec: np.ndarray,
    jnu_pec: np.ndarray,
    jnpy_pec: np.ndarray,
) -> tuple[
    np.ndaray,
    np.ndarray,
    np.ndarray,
    np.ndarray,
    np.ndarray,
    np.ndarray,
    np.ndarray,
    np.ndarray,
    np.ndarray,
]: ...
def func_cython(
    h2vec: np.ndarray,
    w: complex,
    pol: str,
    n: int,
    e1: complex,
    e2: complex,
    r: float,
    roots: np.ndarray,
) -> tuple[float, float]: ...
def eig_eq_cython(
    h2vec: np.ndarray,
    w: complex,
    pol: str,
    n: int,
    e1: complex,
    e2: complex,
    r: float,
    roots: np.ndarray,
) -> tuple[np.ndarray, np.ndarray]: ...



import cmath

import numpy as np
import numpy.testing as npt
import pandas as pd
import scipy.special
from numpy.testing._private.utils import assert_equal

from pymwm.cutoff import Cutoff


def test_cutoffs():
    co = Cutoff(2, 2)
    num_rr = len(co.r_ratios)
    assert co.num_n == 2
    assert co.num_m == 2
    assert len(co.samples) == co.num_n * (2 * co.num_m + 1) * num_rr
    co = Cutoff(2, 1)
    assert co.num_m == 1
    assert len(co.samples) == co.num_n * (2 * co.num_m + 1) * num_rr


def test_cutoffs_TE_r1_0():
    co = Cutoff(3, 3)
    num_rr = len(co.r_ratios)
    assert co.num_n == 3
    assert co.num_m == 3
    assert len(co.samples) == co.num_n * (2 * co.num_m + 1) * num_rr
    for n in range(3):
        bessels = scipy.special.jnp_zeros(n, 3)
        for m in range(1, 4):
            s = co.samples.query(f"pol=='E' and n=={n} and m=={m} and irr==0").iloc[0]
            print(s["val"], bessels[m - 1])
            assert s["val"] == bessels[m - 1]


def test_cutoffs_TM_r1_0():
    co = Cutoff(3, 3)
    s = co.samples.query("pol=='M' and n==0 and m==1 and irr==0").iloc[0]
    assert s["val"] == 0.0
    bessels = scipy.special.jn_zeros(0, 4)
    for m in range(2, 5):
        s = co.samples.query(f"pol=='M' and n==0 and m=={m} and irr==0").iloc[0]
        print(s["val"], bessels[m - 2])
        assert s["val"] == bessels[m - 2]
    for n in range(1, 3):
        bessels = scipy.special.jn_zeros(n, 4)
        for m in range(1, 5):
            s = co.samples.query(f"pol=='M' and n=={n} and m=={m} and irr==0").iloc[0]
            print(s["val"], bessels[m - 1])
            assert s["val"] == bessels[m - 1]


def test_cutoffs_TE():
    co = Cutoff(2, 3)
    num_rr = len(co.r_ratios)
    assert co.num_n == 2
    assert co.num_m == 3
    assert len(co.samples) == co.num_n * (2 * co.num_m + 1) * num_rr
    a = 0.1
    c = 0.5
    r_ratio = a / c
    k0 = 0.3
    eps = 1
    kc = co(("E", 1, 2), r_ratio) / c
    beta = (1 + 1j) * cmath.sqrt(-0.5j * (eps * k0 ** 2 - kc ** 2))
    val = beta ** 2
    print(val)
    npt.assert_allclose(val, -98.35032331623853 + 0j, rtol=1e-6)


def test_cutoffs_cython():
    co = Cutoff(3, 3)
    num_rrs = len(co.r_ratios)
    df1 = co.cutoffs()
    df2 = co.cutoffs_numpy()
    for pol, m_end in [("M", 4), ("E", 4)]:
        for n in range(3):
            for m in range(1, m_end):
                for irr in range(num_rrs):
                    val1 = df1[
                        (df1["pol"] == pol)
                        & (df1["n"] == n)
                        & (df1["m"] == m)
                        & (df1["irr"] == irr)
                    ]["val"].iloc[0]
                    val2 = df2[
                        (df2["pol"] == pol)
                        & (df2["n"] == n)
                        & (df2["m"] == m)
                        & (df2["irr"] == irr)
                    ]["val"].iloc[0]
                    try:
                        npt.assert_almost_equal(val1, val2)
                    except Exception as e:
                        print(pol, n, m, irr, num_rrs, val1, val2)
                        raise e


def test_cutoffs_samples(num_regression):
    df = Cutoff(16, 8).samples
    d = {}
    for pol, m_end in [("M", 10), ("E", 9)]:
        for n in range(16):
            for m in range(1, m_end):
                df1 = df[(df["pol"] == pol) & (df["n"] == n) & (df["m"] == m)]
                d[f"{pol}_{n}_{m}_rr"] = df1["rr"].to_numpy()
                d[f"{pol}_{n}_{m}_val"] = df1["val"].to_numpy()
    num_regression.check(d)



from __future__ import annotations

import numpy as np
import numpy.testing as npt

import pymwm

params: dict = {
    "core": {"shape": "coax", "r": 0.15, "ri": 0.1, "fill": {"RI": 1.0}},
    "clad": {"book": "Au", "page": "Stewart-DLF", "bound_check": False},
    "bounds": {"wl_max": 5.0, "wl_min": 1.0, "wl_imag": 50.0},
    "modes": {"wl_max": 20.0, "wl_min": 1.0, "wl_imag": 50.0, "num_n": 6, "num_m": 2},
}


def test_attributes():
    wg = pymwm.create(params)
    assert wg.r == 0.15
    assert wg.ri == 0.1
    w = 2 * np.pi / 5.0
    assert wg.fill(w) == 1.0
    drude_lorentz = -1272.37592771801 + 351.25089220304176j
    npt.assert_almost_equal(wg.clad(w), drude_lorentz)



import unittest

import numpy as np

import pymwm


class TestSlit(unittest.TestCase):
    def setUp(self):
        self.params = {
            "core": {"shape": "slit", "size": 0.3, "fill": {"RI": 1.0}},
            "clad": {"book": "Au", "page": "Stewart-DLF"},
            "bounds": {"wl_max": 5.0, "wl_min": 1.0, "wl_imag": 50.0},
            "modes": {"wl_max": 5.0, "wl_min": 1.0, "wl_imag": 50.0, "num_n": 6},
        }
        self.pec = {"PEC": True}

    def test_attributes(self):
        params = self.params.copy()
        wg = pymwm.create(params)
        self.assertEqual(wg.r, 0.3)
        w = 2 * np.pi / 5.0
        self.assertEqual(wg.fill(w), 1.0)
        drude_lorentz = -1272.37592771801 + 351.25089220304176j
        self.assertAlmostEqual(wg.clad(w), drude_lorentz)

    def test_Yab_pec(self):
        params = self.params.copy()
        params["clad"] = self.pec
        wg = pymwm.create(params)
        w = 2 * np.pi / 5.0
        alpha1 = ("E", 1, 1)
        h1 = wg.beta(w, alpha1)
        a1, b1 = wg.coef(h1, w, alpha1)
        alpha2 = ("M", 1, 1)
        h2 = wg.beta(w, alpha2)
        a2, b2 = wg.coef(h2, w, alpha2)
        s1 = 0
        s2 = 1
        l1 = l2 = 0
        n1 = n2 = 1
        m1 = m2 = 1
        self.assertAlmostEqual(wg.norm(w, h1, alpha1, a1, b1), 1.0)
        self.assertAlmostEqual(wg.norm(w, h2, alpha2, a2, b2), 1.0)
        self.assertAlmostEqual(wg.Y(w, h1, alpha1, a1, b1), h1 / w)
        self.assertEqual(wg.Y(w, h2, alpha2, a2, b2), w / h2)
        self.assertEqual(
            wg.Yab(w, h1, s1, l1, n1, m1, a1, b1, h1, s1, l1, n1, m1, a1, b1), h1 / w
        )
        self.assertEqual(
            wg.Yab(w, h2, s2, l2, n2, m2, a2, b2, h2, s2, l2, n2, m2, a2, b2), w / h2
        )
        self.assertEqual(
            wg.Yab(w, h1, s1, l1, n1, m1, a1, b1, h2, s2, l2, n2, m2, a2, b2), 0.0
        )

    def test_Yab_no_loss(self):
        params = self.params.copy()
        params["clad"] = {"book": "Au", "page": "Stewart-DLF", "im_factor": 0.0}
        wg = pymwm.create(params)
        w = 2 * np.pi / 2.0
        alpha1 = ("E", 1, 1)
        h1 = wg.beta(w, alpha1)
        a1, b1 = wg.coef(h1, w, alpha1)
        alpha2 = ("M", 1, 1)
        h2 = wg.beta(w, alpha2)
        a2, b2 = wg.coef(h2, w, alpha2)
        s1 = 0
        s2 = 1
        l1 = l2 = 0
        n1 = n2 = 1
        m1 = m2 = 1
        print(a1, b1, a2, b2)
        self.assertAlmostEqual(wg.norm(w, h1, alpha1, a1, b1), 1.0)
        self.assertAlmostEqual(wg.norm(w, h2, alpha2, a2, b2), 1.0)
        self.assertAlmostEqual(
            wg.Yab(w, h1, s1, l1, n1, m1, a1, b1, h1, s1, l1, n1, m1, a1, b1),
            wg.Y(w, h1, alpha1, a1, b1),
        )
        self.assertAlmostEqual(
            wg.Yab(w, h2, s2, l2, n2, m2, a2, b2, h2, s2, l2, n2, m2, a2, b2),
            wg.Y(w, h2, alpha2, a2, b2),
        )
        print(wg.Yab(w, h1, s1, l1, n1, m1, a1, b1, h2, s2, l2, n2, m2, a2, b2))
        self.assertAlmostEqual(
            wg.Yab(w, h1, s1, l1, n1, m1, a1, b1, h2, s2, l2, n2, m2, a2, b2), 0.0
        )

    def test_Yab_with_loss(self):
        params = self.params.copy()
        wg = pymwm.create(params)
        w = 2 * np.pi / 2.0
        alpha1 = ("E", 1, 1)
        h1 = wg.beta(w, alpha1)
        a1, b1 = wg.coef(h1, w, alpha1)
        alpha2 = ("M", 1, 1)
        h2 = wg.beta(w, alpha2)
        a2, b2 = wg.coef(h2, w, alpha2)
        s1 = 0
        s2 = 1
        l1 = l2 = 0
        n1 = n2 = 1
        m1 = m2 = 1
        self.assertAlmostEqual(wg.norm(w, h1, alpha1, a1, b1), 1.0)
        self.assertAlmostEqual(wg.norm(w, h2, alpha2, a2, b2), 1.0)
        print(wg.Yab(w, h1, s1, l1, n1, m1, a1, b1, h2, s2, l2, n2, m2, a2, b2))
        self.assertEqual(
            wg.Yab(w, h1, s1, l1, n1, m1, a1, b1, h2, s2, l2, n2, m2, a2, b2), 0.0
        )

    def test_Yaa_and_Y(self):
        params = self.params.copy()
        wg = pymwm.create(params)
        w = 2 * np.pi
        for i, alpha in enumerate(wg.alpha_all):
            h = wg.beta(w, alpha)
            a, b = wg.coef(h, w, alpha)
            s = wg.s_all[i]
            dir = wg.l_all[i]
            n = wg.n_all[i]
            m = wg.m_all[i]
            Yab = wg.Yab(w, h, s, dir, n, m, a, b, h, s, dir, n, m, a, b)
            Y = wg.Y(w, h, alpha, a, b)
            print(Yab)
            print(Y)
            self.assertAlmostEqual(Yab, Y)


if __name__ == "__main__":
    unittest.main()



from __future__ import annotations

import numpy as np

def eig_eq_with_jac(
    h2: np.ndarray,
    w: complex,
    pol: str,
    n: int,
    e1: complex,
    e2: complex,
    r: float,
    ri: float,
    roots: np.ndarray,
) -> tuple[np.ndarray, np.ndarray]: ...
def eig_eq_for_min(
    h2: np.ndarray,
    w: complex,
    pol: str,
    n: int,
    e1: complex,
    e2: complex,
    r: float,
    ri: float,
    roots: np.ndarray,
) -> float: ...
def eig_eq_for_min_with_jac(
    h2: np.ndarray,
    w: complex,
    pol: str,
    n: int,
    e1: complex,
    e2: complex,
    r: float,
    ri: float,
    roots: np.ndarray,
) -> tuple[float, np.ndarray]: ...
def eig_eq(
    h2: np.ndarray,
    w: complex,
    pol: str,
    n: int,
    e1: complex,
    e2: complex,
    r: float,
    ri: float,
    roots: np.ndarray,
) -> np.ndarray: ...
def eig_eq_u_with_jac(
    u: np.ndarray,
    w: complex,
    pol: str,
    n: int,
    e1: complex,
    e2: complex,
    r: float,
    ri: float,
    roots: np.ndarray,
) -> tuple[np.ndarray, np.ndarray]: ...
def eig_eq_u(
    u: np.ndarray,
    w: complex,
    pol: str,
    n: int,
    e1: complex,
    e2: complex,
    r: float,
    ri: float,
    roots: np.ndarray,
) -> np.ndarray: ...
def eig_eq_u_for_min(
    u: np.ndarray,
    w: complex,
    pol: str,
    n: int,
    e1: complex,
    e2: complex,
    r: float,
    ri: float,
    roots: np.ndarray,
) -> float: ...

# def func_cython(
#     h2vec: np.ndarray,
#     w: complex,
#     pol: str,
#     n: int,
#     e1: complex,
#     e2: complex,
#     r: float,
#     ri: float,
#     roots: np.ndarray,
# ) -> tuple[float, float]: ...

def props_cython(
    w: complex,
    f: float,
    ri: float,
    s_all: np.ndarray,
    n_all: np.ndarray,
    hs: np.ndarray,
    e1: complex,
    e2: complex,
) -> tuple[
    np.ndarray,
    np.ndarray,
    np.ndarray,
    np.ndarray,
    np.ndarray,
    np.ndarray,
    np.ndarray,
    np.ndarray,
    np.ndarray,
    np.ndarray,
    np.ndarray,
    np.ndarray,
    np.ndarray,
    np.ndarray,
    np.ndarray,
    np.ndarray,
    np.ndarray,
    np.ndarray,
    np.ndarray,
    np.ndarray,
    np.ndarray,
    np.ndarray,
    np.ndarray,
    np.ndarray,
    np.ndarray,
]: ...



import numpy as np
import numpy.testing as npt

from pymwm.coax.samples import Samples

params: dict = {
    "core": {"shape": "coax", "r": 0.15, "ri": 0.1, "fill": {"RI": 1.0}},
    "clad": {"book": "Au", "page": "Stewart-DLF", "bound_check": False},
    "modes": {
        "wl_max": 20.0,
        "wl_min": 1.0,
        "wl_imag": 50.0,
        "dw": 1.0 / 64,
        "num_n": 6,
        "num_m": 2,
    },
}


def test_attributes():
    size: float = params["core"]["r"]
    fill = params["core"]["fill"]
    clad = params["clad"]
    p = params["modes"]
    size2: float = params["core"]["ri"]
    wg = Samples(size, fill, clad, p, size2)
    ind_w_min = int(np.floor(2 * np.pi / p["wl_max"] / p["dw"]))
    ind_w_max = int(np.ceil(2 * np.pi / p["wl_min"] / p["dw"]))
    ind_w_imag = int(np.ceil(2 * np.pi / p["wl_imag"] / p["dw"]))
    ws = np.arange(ind_w_min, ind_w_max + 1) * p["dw"]
    wis = -np.arange(ind_w_imag + 1) * p["dw"]
    npt.assert_equal(wg.ws, ws)
    npt.assert_equal(wg.wis, wis)


def test_beta2_pec(num_regression):
    size: float = params["core"]["r"]
    fill = params["core"]["fill"]
    clad = params["clad"]
    p = params["modes"]
    size2: float = params["core"]["ri"]
    wg = Samples(size, fill, clad, p, size2)
    w = 2 * np.pi / 5.0
    num_regression.check({n: wg.beta2_pec(w, n).real for n in range(6)})


def test_compare_with_cylinder():
    import pymwm.cylinder

    size: float = params["core"]["r"]
    fill = params["core"]["fill"]
    clad = params["clad"]
    p = params["modes"]
    size2 = 1e-5
    wg1 = Samples(size, fill, clad, p, size2)
    wg2 = pymwm.cylinder.samples.Samples(size, fill, clad, p)
    w = 2 * np.pi / 20.0

    val1 = wg1.beta2_pec(w, 0)
    val2 = wg2.beta2_pec(w, 0)
    try:
        npt.assert_allclose(val1[1:3], val2[:2], rtol=0.1)
    except Exception as e:
        print(f"n={0}")
        print(val1[1:3])
        print(val2[:2])
        raise e
    try:
        npt.assert_allclose(val1[3:], val2[3:], rtol=0.1)
    except Exception as e:
        print(f"n={0}")
        print(val1[3:])
        print(val2[3:])
        raise e
    val1, success1 = wg1.beta2_w_min(0)
    val2, success2 = wg2.beta2_w_min(0)
    try:
        npt.assert_array_equal(success1[1:3], success2[:2])
        npt.assert_allclose(val1[1:3], val2[:2], rtol=0.001, atol=0.001)
    except Exception as e:
        print(f"n={0}")
        print("coax", success1[1:3], val1[1:3])
        print("cylinder", success2[:2], val2[:2])
        raise e
    try:
        npt.assert_array_equal(success1[3:], success2[3:])
        npt.assert_allclose(val1[3:], val2[3:], rtol=0.001)
    except Exception as e:
        print(f"n={0}")
        print("coax", success1[3:], val1[3:])
        print("cylinder", success2[3:], val2[3:])
        raise e
    for n in range(1, 6):
        try:
            val1 = wg1.beta2_pec(w, n)
            val2 = wg2.beta2_pec(w, n)
            npt.assert_allclose(val1, val2, rtol=0.1)
        except Exception as e:
            print(f"n={n}")
            print(val1)
            print(val2)
            raise e
        try:
            val1 = wg1.beta2_w_min(n)
            val2 = wg2.beta2_w_min(n)
            npt.assert_allclose(val1, val2, rtol=0.001)
        except Exception as e:
            print(f"n={n}")
            print("coax", val1)
            print("cylinder", val2)
            raise e


def test_beta2_w_min(num_regression):
    size: float = params["core"]["r"]
    fill = params["core"]["fill"]
    clad = params["clad"]
    p = params["modes"]
    size2: float = params["core"]["ri"]
    wg = Samples(size, fill, clad, p, size2)
    d = {}
    for n in range(5):
        val = wg.beta2_w_min(n)
        d[f"{n}_real"] = val[0].real
        d[f"{n}_imag"] = val[0].imag
        d[f"{n}_success"] = val[1]
    num_regression.check(d)
